from PIL import Image

from .pixel import Pixel

import tkinter as tk
from tkinter import Canvas
from tkinter import Frame
from tkinter import PhotoImage


def rgb_to_hex(r, g, b):
    return '#%02x%02x%02x' % (r, g, b)

class SimplePicture(object):
    def __init__(self, *args):
        if len(args) == 2:
            self.width = args[0]
            self.height = args[1]
            self.filename = None
            self.pixels = [Pixel(x, y, red=255, green=255, blue=255) 
                        for x in range(self.width) 
                        for y in range(self.height)]
        elif len(args) == 1:
            self.filename = args[0]
            self.pixels = []
            self.decode_picture_file(self.filename)
        else:
            raise Exception('Could not create Picture. Please specify either a width and height or a filename.')
    
    def getWidth(self):
        return self.width

    def getHeight(self):
        return self.height

    def getPixels(self):
        return self.pixels
    
    def getPixel(self, x, y):
        if x < 0 or x >= self.width:
            raise IndexError('Error in call to getPixel(%d, %d). x-value is out of range.' % (x, y))
        if y < 0 or y >= self.height:
            raise IndexError('Error in call to getPixel(%d, %d). y-value is out of range.' % (x, y))

        return self.pixels[y * self.width + x]

    def decode_picture_file(self, filename):
        #if not filename.endswith('.png'):
        #    raise Exception(filename + ' is not a .png file. Please select a png file.')

        self.img = Image.open(filename)
        self.width, self.height = self.img.size

        pixels = self.img.load()
        self.pixels = [Pixel(x, y, p[0], p[1], p[2])
            for y in range(self.height)
            for x in range(self.width)
            for p in [pixels[x, y]]]
        
    def show(self):
        self.explore(show_mode=True)

    def explore(self, show_mode=False):
        window = tk.Tk()
        window.wm_title(self.filename.split('/')[-1] if self.filename else 'Your picture!')
        canvas = Canvas(window, width=self.width, height=self.height, 
                        highlightthickness=1, highlightbackground='black',
                        relief='ridge')
        
        img = PhotoImage(master=canvas, width=self.width, height=self.height)
        canvas.create_image((self.width // 2, self.height // 2), image=img, state='normal')
        canvas.grid(row=1, column=0)
        
        # Create big string that contains rows of colors like this 
        # row 1: '{color color color}
        # row 2: {color color color}'
        # etc.
        line = ''
        for y in range(self.height):
            line += '{' + ' '.join([
                rgb_to_hex(self.getPixel(x, y).getRed(), 
                           self.getPixel(x, y).getGreen(), 
                           self.getPixel(x, y).getBlue())
                for x in range(self.width)]) + '} '

        # Paint the string onto the image.
        img.put(line, to=(0, 0, self.width, self.height))
        
        if not show_mode:
            frame = Frame(window)
            frame.grid(row=0, column=0)
        
            text = tk.Text(frame, height=2, width=30, relief='solid', font=('Consolas', 10))
            text.tag_configure('center', justify='center')
            text.grid(row=0, column=0)
            text.config(state='disabled')
            
            pix_label = tk.Text(frame, height=1, width=15, relief='solid', 
                                borderwidth=0,
                                highlightthickness=0,
                                font=('Consolas', 10))
            pix_label.grid(row=0, column=1, padx=10)
            pix_label.insert(tk.END, 'Selected Color:')
            pix_label.config(state='disabled')
            
            pix_color = Canvas(frame, width=20, height=20, bd=0,
                               borderwidth=1,
                               highlightthickness=1, relief='ridge')
            pix_color.grid(row=0, column=2, padx=10)
            
            x_line = canvas.create_line(-1, -1, -1, -1)
            y_line = canvas.create_line(-1, -1, -1, -1)
            
            def onclick(event):
                nonlocal x_line, y_line
                if event.x < 0 or event.x >= self.width or event.y < 0 or event.y >= self.height:
                    return
                
                color = self.getPixel(event.x, event.y).getColor()
                
                text.config(state='normal')
                text.delete('1.0', tk.END)
                text.insert(tk.END,'X: ' + str(event.x) + ', Y: ' + str(event.y) + '\n')
                text.insert(tk.END, 'R: {}, G: {}, B: {}'.format(*color.getRGB()))
                text.tag_add('center', '1.0', 'end')
                text.config(state='disabled')
                
                # Draw crosshair.
                canvas.delete(x_line, y_line)
                x_line = canvas.create_line(event.x-4, event.y, event.x+4, event.y, 
                                            fill='yellow')
                y_line = canvas.create_line(event.x, event.y-4, event.x, event.y+4, 
                                            fill='yellow')
                
                # Set pixel preview canvas to the color of the selected pixel.
                pix_color.create_rectangle(-1, -1, 
                                           pix_color.winfo_width(), 
                                           pix_color.winfo_height(),
                                           fill='#%02x%02x%02x' % color.getRGB())
            
            canvas.bind('<Button-1>', onclick)
        
        quit_code = False
        def close_window():
            nonlocal quit_code
            quit_code = True
            window.destroy()
        window.protocol('WM_DELETE_WINDOW', close_window)
        
        window.focus_set()
        
        # Present window.
        while not quit_code:
            window.update()

    def write(self, filename):
        image = Image.new('RGB', (self.width, self.height))
        pixels = image.load()
        for y in range(self.height):
            for x in range(self.width):
                p = self.getPixel(x, y)
                pixels[x, y] = (p.red, p.green, p.blue)

        image.save(filename)
